package xyz.zhouxy.plusone.oss;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.List;
import java.util.Objects;

import org.csource.common.MyException;
import org.csource.fastdfs.ClientGlobal;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import xyz.zhouxy.plusone.oss.FastDFSProperties.ConnectionPool;

@Configuration
@EnableConfigurationProperties(FastDFSProperties.class)
@ConditionalOnClass(FastDFSUtil.class)
public class FastDFSAutoConfig {

    @Bean
    @SuppressWarnings("all")
    FastDFSUtil fastDFSUtil(FastDFSProperties props) throws IOException, FastDFSException {

        List<String> trackerServerStrList = props.getTrackerServers();
        if (CollectionUtils.isEmpty(trackerServerStrList)) {
            throw new FastDFSException(
                    String.format("configure item %s is required - ", ClientGlobal.PROP_KEY_TRACKER_SERVERS));
        }
        try {
            InetSocketAddress[] trackerServers = trackerServerStrList.stream()
                    .map(trackerServer -> {
                        String[] hostPort = trackerServer.trim().split(":");
                        String host = hostPort[0].trim();
                        int port = Integer.parseInt(hostPort[1].trim());
                        return new InetSocketAddress(host, port);
                    })
                    .toArray(InetSocketAddress[]::new);
            ClientGlobal.initByTrackers(trackerServers);

            var connectTimeoutInSecondsConf = props.getConnectTimeoutInSeconds();
            if (connectTimeoutInSecondsConf != null) {
                ClientGlobal.setG_connect_timeout(connectTimeoutInSecondsConf * 1000);
            }
            var networkTimeoutInSecondsConf = props.getNetworkTimeoutInSeconds();
            if (networkTimeoutInSecondsConf != null) {
                ClientGlobal.setG_network_timeout(networkTimeoutInSecondsConf * 1000);
            }
            var charsetConf = props.getCharset();
            if (StringUtils.hasText(charsetConf)) {
                ClientGlobal.setG_charset(charsetConf);
            }
            var httpAntiStealTokenConf = props.getHttpAntiStealToken();
            if (httpAntiStealTokenConf != null) {
                ClientGlobal.setG_anti_steal_token(httpAntiStealTokenConf);
            }
            var httpSecretKeyConf = props.getHttpSecretKey();
            if (StringUtils.hasText(httpSecretKeyConf)) {
                ClientGlobal.setG_secret_key(httpSecretKeyConf);
            }
            var httpTrackerHttpPortConf = props.getHttpTrackerHttpPort();
            if (httpTrackerHttpPortConf != null) {
                ClientGlobal.setG_tracker_http_port(httpTrackerHttpPortConf);
            }

            ConnectionPool connectionPool = props.getConnectionPool();
            var poolEnabled = Objects.nonNull(connectionPool)
                    && Boolean.TRUE.equals(connectionPool.getEnabled());

            if (poolEnabled) {
                var poolMaxCountPerEntry = connectionPool.getMaxCountPerEntry();
                if (poolMaxCountPerEntry != null) {
                    ClientGlobal.g_connection_pool_max_count_per_entry = poolMaxCountPerEntry;
                }
                var poolMaxIdleTime = connectionPool.getMaxIdleTime();
                if (poolMaxIdleTime != null) {
                    ClientGlobal.g_connection_pool_max_idle_time = poolMaxIdleTime * 1000;
                }
                var poolMaxWaitTimeInMS = connectionPool.getMaxWaitTimeInMs();
                if (poolMaxWaitTimeInMS != null) {
                    ClientGlobal.g_connection_pool_max_wait_time_in_ms = poolMaxWaitTimeInMS;
                }
            }
            return new FastDFSUtil();
        } catch (MyException e) {
            throw new FastDFSException(e);
        }
    }
}
