package xyz.zhouxy.plusone.system.domain.model.account;

import java.util.Objects;
import java.util.regex.Pattern;

import cn.hutool.core.util.DesensitizedUtil;
import xyz.zhouxy.plusone.commons.annotation.StaticFactoryMethod;
import xyz.zhouxy.plusone.commons.constant.PatternConsts;

/**
 * 值对象：电子邮箱地址
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
public class Email extends Principal {

    public static final Pattern REGEX = PatternConsts.EMAIL;

    private Email(String email) {
        super(REGEX);
        if (email == null) {
            throw new IllegalArgumentException("邮箱地址不能为空");
        }
        this.value = email;
        if (!isValid()) {
            throw new IllegalArgumentException("邮箱地址格式错误");
        }
    }

    @StaticFactoryMethod(Email.class)
    public static Email of(String email) {
        return new Email(email);
    }

    @StaticFactoryMethod(Email.class)
    public static Email ofNullable(String email) {
        return Objects.nonNull(email) ? new Email(email) : null;
    }

    /**
     * 脱敏后的数据
     */
    public String safeValue() {
        return DesensitizedUtil.email(this.value);
    }

    @Override
    public String toString() {
        return this.safeValue();
    }
}
